% precompute_outlier_probabilities 
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/

function [Prob_outliers,M]=precompute_outlier_probabilities(N,pd)
%computes the probability of n outliers: n=1..N given the detection
%probability pd
%N:total number of model points
%pd:probability of detection
M=N-ceil(pd*N); %number of outliers

cont0=1;
for n=1:N
    O=[];
    I=[];
    cont=1;
    for r=0:n
        w=n-r;
        if N-M-w < 0 | M-r<0
            drawn(cont0).P(cont)=0;
        else
            num=factorial(M)*factorial(N-M)*factorial(N-n);
            den=factorial(M-r)*factorial(N-M-w)*factorial(N);
            drawn(cont0).P(cont)=num/den;
        end
        O(cont)=r;
        I(cont)=w;
        cont=cont+1;
    end
    
    drawn(cont0).P=drawn(cont0).P/sum(drawn(cont0).P);
    drawn(cont0).O=O;
    drawn(cont0).I=I;
    cont0=cont0+1;
end

Prob_outliers=drawn;
