% recompute_pose 
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/


function [Pose_new,Sigma_new]=recompute_pose(Pose_ini,Sigma_ini,covariance_sensor,Xi,Ui,Ui_observed,A)

wx=Pose_ini(1); wy=Pose_ini(2); wz=Pose_ini(3);
Tx_ini=Pose_ini(4); Ty_ini=Pose_ini(5); Tz_ini=Pose_ini(6);
Rini=rodrigues2mat(wx,wy,wz);
Tini=[Tx_ini,Ty_ini,Tz_ini]';

xworld=Xi(1); yworld=Xi(2); zworld=Xi(3);  
Xworld=[xworld,yworld,zworld]';

Xcam=Rini*Xworld+Tini;
xcam=Xcam(1); ycam=Xcam(2); zcam=Xcam(3);

%compute Jacobians
JRT=Jacobian_RT_rodrigues(wx,wy,wz);
Jxyz=Jacobian_xyz(xworld,yworld,zworld);
Juv=Jacobian_uv(A,xcam,ycam,zcam);
J=Juv*Jxyz*JRT;


%Kalman filter
innovation=Ui_observed'-Ui';
S=J*Sigma_ini*J'+covariance_sensor; 
K=Sigma_ini*J'*inv(S); 
Pose_new=Pose_ini+K*innovation;
Sigma_new=(eye(6)-K*J)*Sigma_ini;

