% recursive_function 
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/

function [tree,terminated]=recursive_function(A,Pose_ini,Sigma_ini,Sigma_sensor,SENSOR_NOISE,Xw,U,Model_points_remaining,tree,parent,threshold_mah,threshold_eucl,Ureal,Prob_outliers,MIN_ERROR_THRESHOLD,TERMINATION_THRESHOLD,terminated,KICKOUT_THRESHOLD)

if terminated==1
    return;
end

elaptime=toc;
if elaptime>25
    fprintf('TOO MUCH TIME!!!\n');
    return;
end

idx2consider=Model_points_remaining;
last_node=tree.node(end);
n3d=size(Xw,1);
MAX_VALID_MATCHES=4;
MAX_DEPTH=size(Xw,1); %max depth of the tree

nvalid_matches_upto_node=num_valid_matches_upto_node(tree);
depth_node=tree.depth(last_node);
if nvalid_matches_upto_node<MAX_VALID_MATCHES &  depth_node<=MAX_DEPTH
    %case: not estabilized --> count number of outliers
    %count number of outliers and if Numberourliers*errorourlier <
    %min_error_opt do not stop the branch. Otherwise stop the branch
    ek=early_kickout(tree,Prob_outliers,KICKOUT_THRESHOLD);
    if ek
        stop_this_branch=1;
    else
        err_only_outliers=estimate_minimum_error_just_outliers(n3d,tree,SENSOR_NOISE);
        if err_only_outliers<tree.min_error_optimal
            %if error < min_error --> continue projecting
            [Uproj,covariance_uv,covariance_xyz,covariance_RT]=project_with_variance(A,Pose_ini,Sigma_ini,Sigma_sensor,Xw,U); 
            [candidates1,Ui1,Xi1,idx2consider1,idx1]=find_candidate_points(Xw,U,Uproj,covariance_uv,threshold_mah,threshold_eucl,idx2consider);
            stop_this_branch=0;
            if size(candidates1,2)==0
                stop_this_branch=1;
            end
        else
            stop_this_branch=1;
        end
    end
elseif nvalid_matches_upto_node==MAX_VALID_MATCHES 
    %case: already estabilized. The error cannot be improved
    Uproj=project_points2(A,Pose_ini,Xw);
    [err_opt,percentage_outliers,num_inliers]=estimate_minimum_reprojection_error1(U,Uproj,tree,last_node,SENSOR_NOISE);
    tree.error(last_node)=err_opt;
    if err_opt<tree.min_error_optimal
        tree.min_error_optimal=err_opt;
    end
    stop_this_branch=1;
    
    %check termination
    if num_inliers>=TERMINATION_THRESHOLD & err_opt<MIN_ERROR_THRESHOLD
        terminated=1;
    end  
else
    stop_this_branch=1;
end

if terminated==1
    return;
end

isaleaf=stop_this_branch;
if isaleaf %leaf: no candidate or max depth of the tree
    node_leaf=tree.node(end);
    %assign a number to the leaf
    tree.leaves=tree.leaves+1;
    tree.leaf(node_leaf)=tree.leaves;

else %the node is not a leaf
    candidates1(end+1)=[0]; %flag for the outlier case!!!!!!!!!!!!!!!!!!!
    ncandidates=size(candidates1,2);

    for i=1:ncandidates
        idx1_obs=candidates1(i);
        %update tree
        new_node=tree.node(end)+1;
        tree.node(new_node)=new_node;
        tree.parent(new_node)=parent;
        %children
        tree.children(new_node,1)=0; %initialize children for "new_node"
        tree.depth(new_node)=tree.depth(parent)+1;
        %fill children in "new_node's parent"
        nchilds=sum(tree.children(parent,:)~=0);
        tree.children(parent,nchilds+1)=new_node;
        tree.match(new_node,:)=[idx1,candidates1(i)];
        %ancestors
        nancestors_parent=tree.nancestors(parent);
        tree.ancestors(new_node,1:nancestors_parent)=[tree.ancestors(parent,1:nancestors_parent)];
        tree.nancestors(new_node)=nancestors_parent+1;
        tree.ancestors(new_node,nancestors_parent+1)=parent;
        %valid ancestors
        if tree.match(parent,2)==0 %parent is outlier-->no valid ancestor
            nvalid_ancestors_parent=tree.nvalid_ancestors(parent);
            tree.nvalid_ancestors(new_node)=nvalid_ancestors_parent;
            tree.valid_ancestors(new_node,:)=tree.valid_ancestors(parent,:);
        else
            nvalid_ancestors_parent=tree.nvalid_ancestors(parent);
            tree.nvalid_ancestors(new_node)=nvalid_ancestors_parent+1;
            tree.valid_ancestors(new_node,:)=tree.valid_ancestors(parent,:);
            tree.valid_ancestors(new_node,nvalid_ancestors_parent+1)=parent;
        end

        if idx1_obs~=0 %there is candidate for the point idx
            Ui1_observed=U(idx1_obs,:);
            [Pose_new1,Sigma_new1]=recompute_pose(Pose_ini,Sigma_ini,Sigma_sensor,Xi1,Ui1,Ui1_observed,A); %Aqui hi ha el kalman!!!!!!!
            tree.pose(new_node,:)=Pose_new1;
            parent_new=new_node;
            [tree,terminated]=recursive_function(A,Pose_new1,Sigma_new1,Sigma_sensor,SENSOR_NOISE,Xw,U,idx2consider1,tree,parent_new,threshold_mah,threshold_eucl,Ureal,Prob_outliers,MIN_ERROR_THRESHOLD,TERMINATION_THRESHOLD,terminated,KICKOUT_THRESHOLD);
        else %outlier
            tree.pose(new_node,:)=Pose_ini;
            parent_new=new_node;
            [tree,terminated]=recursive_function(A,Pose_ini,Sigma_ini,Sigma_sensor,SENSOR_NOISE,Xw,U,idx2consider1,tree,parent_new,threshold_mah,threshold_eucl,Ureal,Prob_outliers,MIN_ERROR_THRESHOLD,TERMINATION_THRESHOLD,terminated,KICKOUT_THRESHOLD);
        end
    end % for i=1:ncandidates
end %if isaleaf
