% generate_2d_noisy_input_data2 
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/

function [U,Ureal,Rtrue,Ttrue,Pose_true,Xw_no_occluded]=generate_2D_noisy_input_data2(Xw,P,A,pc,pd,std_noise)

%select one of the poses randomly
nposes=size(P,1);
y = randsample(nposes,1);
Pose_true=P(y,:);

%project points using the selected pose
[U_no_noise,Rtrue,Ttrue,Xc]=project_points2(A,Pose_true,Xw);

%create occluded points
M=size(Xw,1);
num_occluded=M-ceil(pd*M);
y = randsample(M,num_occluded);
idx_no_occluded=[1:M];
idx_no_occluded(y)=[];
Xw_no_occluded=Xw(idx_no_occluded,:);
Ureal=U_no_noise(idx_no_occluded,:);


%add noise
npoints=size(Ureal,1);
for i=1:npoints
    noise=randn(1,2)*std_noise;
    Ureal(i,1)=Ureal(i,1)+noise(1);
    Ureal(i,2)=Ureal(i,2)+noise(2); 
end

%Add more 2d points and randomize its ordering
nnew=floor(M*pd*pc/(1-pc));
U=add_more_2d_points(640,480,nnew,Ureal);
U=randomize_vectors(U);

