% generate_pose_priors 
%
%       Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua
%       Pose Priors for Simultaneously Solving Alignment and Correspondence
%       European Conference on Computer Vision (ECCV), 2008
%
% Copyright (C) <2009>  <Francesc Moreno-Noguer, Vincent Lepetit, Pascal Fua>
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Francesc Moreno-Noguer, Institut de Robtica i Informtica Industrial (CSIC/UPC), May 2009.
% fmoreno@iri.upc.edu, http://www-iri.upc.es/people/fmoreno/

function [Poses,Prob,x1,x2,Mu,Cov,A]=generate_pose_priors(Xw,num_gaussians)


%1.-Define intrinsic camera parameters------------------------
fx=800; fy=800; %focal length in pixels
u0=320; v0=240; %we assume a 640x480 image
A=[fx 0  u0;
    0 fy v0;
    0 0  1];

%2.-Center the model points----------------------------
npoints=size(Xw,1);
Cw=mean(Xw);
Xw=Xw-repmat(Cw,[npoints,1]);

%3.-generate_poses
zmin=3; zmax=9;
rad1=max(abs(Xw(:))); rad2=zmin; rad3=zmax;
nsamples1=10; nsamples2=1000; nsamples_rotation=1;
rad1=max(abs(Xw(:))); rad2=(zmax+zmin)/2; rad3=1;
[Poses,x1,x2]=generate_poses_torus(nsamples1,nsamples2,nsamples_rotation,rad1,rad2,rad3,Xw,A);

%4.-Compute covariances using Expectation Maximization 
% M. Figueiredo and A.K.Jain, "Unsupervised learning of finite mixture
% models", PAMI 2002
Kmin=num_gaussians;
Kmax=num_gaussians;
[bestk,Prob,Mu,Cov,dl,countf] = mixtures4(Poses',Kmin,Kmax,0,1e-2,0);
